/* Developed for Lenovo Educational Board Pass 3
   © Lenovo Created by Joshua Grosser
   v. 2.0 */

// include the following libraries
#include <WiFi.h>
#include <WebServer.h>
#include <Preferences.h>
// LED Pins
#define LED_YELLOW 16
#define LED_GREEN 15
#define LED_BLUE 18

// WiFi credentials
String ssid = "";
String password = "";

// Create a web server instance on port 80.
WebServer server(80);
Preferences preferences;

void sendResponse()
{
  // html code to be displayed by webserver
  String htmltext = "<!DOCTYPE html><html>\
                     <head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">\
                     <link rel=\"icon\" href=\"data:,\">\
                     <style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\
                     .button { background-color: #E7E7E7; border: none; color: white; padding: 16px 40px;\
                     text-decoration: none; font-size: 30px; margin: 2px; cursor: pointer;}\
                     .button_y {background-color: #FFFF00;}\
                     .button_g {background-color: #4CAF50;}\
                     .button_b {background-color: #008CBA;}</style></head>\
                     <body><h1>LENOVO GTC Dos</h1>\
                     <h2>LEDs</h2>";

  // Determine state of Yellow LED.
  if (digitalRead(LED_YELLOW) == HIGH)
  {
    // LED is ON. Provide visual indication and set href to enable turning off.
htmltext += "<a href=\"/Y/0\"><button class=\"button button_y\">Y</button></a>";
  }
  else
  {
    // LED is OFF. Provide visual indication and set href to enable turning on.
htmltext += "<a href=\"/Y/1\"><button class=\"button\">Y</button></a>";
  }

  // Determine state of Green LED.
  if (digitalRead(LED_GREEN) == HIGH)
  {
    // LED is ON. Provide visual indication and set href to enable turning off.
htmltext += "<a href=\"/G/0\"><button class=\"button button_g\">G</button></a>";
  }
  else
  {
    // LED is OFF. Provide visual indication and set href to enable turning on.
htmltext += "<a href=\"/G/1\"><button class=\"button\">G</button></a>";
  }

  // Determine state of Blue LED.
  if (digitalRead(LED_BLUE) == HIGH)
  {
    // LED is ON. Provide visual indication and set href to enable turning off.
    htmltext += "<a href=\"/B/0\"><button class=\"button button_b\">B</button></a>";
  }
  else
  {
    // LED is OFF. Provide visual indication and set href to enable turning on.
htmltext += "<a href=\"/B/1\"><button class=\"button\">B</button></a>";
  }

  htmltext += "</body></html>";

  // Return the response to the client.
  server.send(200, "text/html", htmltext);
}

void handleRoot() 
{
  // Call helper function that builds the HTML content
  // and sends it back to the client.
  sendResponse();
}

void handleYellowLed_Off()
{
  // Turn the Yellow LED off.
  digitalWrite(LED_YELLOW, LOW);
  
  // Send HTTP response.
  sendResponse();
}

// Handler to turn Yellow LED On.
void handleYellowLed_On()
{
  // Turn the Yellow LED on.
  digitalWrite(LED_YELLOW, HIGH);
  
  // Send HTTP response.
  sendResponse();
}

void handleGreenLed_Off()
{
  // Turn the Green LED off.
  digitalWrite(LED_GREEN, LOW);
  
  // Send HTTP response.
  sendResponse();
}

// Handler to turn Green LED On.
void handleGreenLed_On()
{
  // Turn the Green LED on.
  digitalWrite(LED_GREEN, HIGH);
  
  // Send HTTP response.
  sendResponse();
}

void handleBlueLed_Off()
{
  // Turn the Blue LED off.
  digitalWrite(LED_BLUE, LOW);
  
  // Send HTTP response.
  sendResponse();
}

// Handler to turn Blue LED On.
void handleBlueLed_On()
{
  // Turn the Blue LED on.
  digitalWrite(LED_BLUE, HIGH);
  
  // Send HTTP response.
  sendResponse();
}

// get Wifi info
void getWifi()
{
  preferences.begin("credentials", false);
  ssid = preferences.getString("ssid", ""); 
  password = preferences.getString("password", "");
  if (ssid == "" || password == ""){
    Serial.println("No ssid or password saved.");
  }
  else {
    Serial.println("Successfully read WiFi information.");
  }
}

void setup() {
  // set the baud rate on Serial port
  Serial.begin(115200);
  Serial.println("HPE GTC Dos says Hello!");
  getWifi();
  // Initialize LED pins
  pinMode(LED_YELLOW, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);
  digitalWrite(LED_YELLOW, LOW);
  digitalWrite(LED_GREEN, LOW);
  digitalWrite(LED_BLUE, LOW);

  // Establish WiFi connection
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid.c_str(), password.c_str());

  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println("");

  // Initialize and start server
  server.on("/", handleRoot);
  server.on("/Y/0", handleYellowLed_Off);
  server.on("/Y/1", handleYellowLed_On);
  server.on("/G/0", handleGreenLed_Off);
  server.on("/G/1", handleGreenLed_On);
  server.on("/B/0", handleBlueLed_Off);
  server.on("/B/1", handleBlueLed_On);

  server.begin();
  Serial.println("Web server is now running.");
  Serial.println("Type in the IP address above to connect.");
}

void loop() {
  // Start accepting and processing client connections
  server.handleClient();
}
